@@ -1,12 +1,29 @@
-# $Id: dea.merge.R 74 2010-10-05 21:41:48Z Lars $
+# $Id: dea.merge.R 85 2010-11-06 22:18:27Z Lars $
 
 
 dea.merge <- function(X, Y, M, RTS = "vrs", ORIENTATION = "in", 
-    XREF = NULL, YREF = NULL,
-    FRONT.IDX = NULL, TRANSPOSE = FALSE, LP = FALSE, CONTROL=NULL,
-    LPK=NULL)
+    XREF = NULL, YREF = NULL, FRONT.IDX = NULL, TRANSPOSE = FALSE)
 {
 
+   rts <- c("fdh","vrs","drs","crs","irs","irs","add")
+   if ( is.real(RTS) )  {
+      RTStemp <- rts[1+RTS] # the first fdh is number 0
+      RTS <- RTStemp
+   }
+   RTS <- tolower(RTS)
+   if ( !(RTS %in% rts) )  stop(paste("Unknown scale of returns:", RTS))
+
+   orientation <- c("in-out","in","out","graph")
+   if ( is.real(ORIENTATION) )  {
+      ORIENTATION_ <- orientation[ORIENTATION+1]  # "in-out" er nr. 0
+      ORIENTATION <- ORIENTATION_
+   }
+   ORIENTATION <- tolower(ORIENTATION)
+   if ( !(ORIENTATION %in% orientation) ) {
+      stop(paste("Unknown value for ORIENTATION:",ORIENTATION),quote=F)
+   }
+
+
 # Mergers should be measured against the originale technology set,
 # therefore we must use XREF og YREF otherwise it will be Xmerger and
 # Ymerger that determines the technology.
@@ -27,15 +44,13 @@ Ymerger <- M %*% Y
 # Potential gains
 E <- dea(Xmerger, Ymerger, RTS=RTS, ORIENTATION=ORIENTATION, 
     XREF = XREF, YREF = YREF,
-    FRONT.IDX = FRONT.IDX, TRANSPOSE = TRANSPOSE, FAST = TRUE, 
-    LP = LP, CONTROL=CONTROL, LPK=LPK)
+    FRONT.IDX = FRONT.IDX, TRANSPOSE = TRANSPOSE, FAST = TRUE)
 
 
 # Individual efficiencies
 e <- dea(X, Y, RTS=RTS, ORIENTATION=ORIENTATION, 
       XREF = XREF, YREF = YREF, FRONT.IDX = FRONT.IDX, 
-      TRANSPOSE = TRANSPOSE, FAST = TRUE,  
-      LP = LP, CONTROL=CONTROL, LPK=LPK)
+      TRANSPOSE = TRANSPOSE, FAST = TRUE)
 
 
 # Inputs of individual firms projected on efficient frontier and
@@ -61,8 +76,7 @@ if ( ORIENTATION == "in" )  {
 # Pure gains from mergers
 Estar <- dea(Xmerger_proj, Ymerger_proj, RTS=RTS, ORIENTATION=ORIENTATION, 
       XREF = XREF, YREF = YREF, FRONT.IDX = FRONT.IDX, 
-      TRANSPOSE = TRANSPOSE, FAST = TRUE,  
-      LP = LP, CONTROL=CONTROL, LPK=LPK)
+      TRANSPOSE = TRANSPOSE, FAST = TRUE)
 
 
 # Learning effect
@@ -75,8 +89,7 @@ Yharm <- diag(1/rowSums(M)) %*% Ymerger_proj
 # Harmony effect
 HA <- dea(Xharm,Yharm, RTS=RTS, ORIENTATION=ORIENTATION, 
       XREF = XREF, YREF = YREF, FRONT.IDX = FRONT.IDX, 
-      TRANSPOSE = TRANSPOSE, FAST = TRUE,  
-      LP = LP, CONTROL=CONTROL, LPK=LPK) 
+      TRANSPOSE = TRANSPOSE, FAST = TRUE) 
 
 
 # Size effect
